<?php

namespace Drupal\asf;

use Drupal\asf\AsfAction;
use Drupal\asf\AsfSchema;


/**
 * Class AsfSchema.
 */
class AsfCron {

  /**
   *
   */
  function cleanup() {
    $options = array(
      'time' => array(
        'value' => strtotime('NOW - 7 days'),
        'operator' => '<'
      ),
      'status' => ASF_STATUS_EXECUTED
    );
    $executed_actions = AsfSchema::selectActions($options);
    $a = new AsfAction();
    foreach ($executed_actions as $executed_action) {
      $a->delete($executed_action->aid);
    }
  }

  /**
   *
   */
  function selectActions() {
    $options = array(
      'time' => array(
        'value' => REQUEST_TIME,
        'operator' => '<'
      ),
      'status' => ASF_STATUS_PENDING
    );
    $actions = AsfSchema::selectActions($options);
    return $actions;
  }


  /**
   * @param $actions
   * @return array
   */
  function deduceTodos($actions) {
    $todos = array();
    foreach ($actions as $action) {
      $todos[$action->eid] = array(
        'eid' => $action->eid,
        'action' => $action->action
      );
    }
    return $todos;
  }


  /**
   *
   */
  function discardActions($actions) {
    foreach ($actions as $action) {
      $a = new AsfAction();
      $a->dismiss($action);
    }
  }


  /**
   *
   */
  function runCron() {
    $actions = $this->selectActions();
    $todos = $this->deduceTodos($actions);
    $this->discardActions($actions);
    //LATER do this per action as it is finished minor improvement.
    if (!empty($todos)) {
      \Drupal::logger('asf')
        ->notice('Actions that will take place: <pre>' . print_r($todos, TRUE) . '</pre>');
    }
    foreach ($todos as $todo) {
      $this->doSingleCronAction($todo);
    }
  }


  /**
   * Do a single Action.
   * $todo['eid'] = entity_id
   * $todo['action'] = publish|unpublish
   */
  function doSingleCronAction($todo) {
    //$entity = \Drupal\node\Entity::load($todo['eid']);
//    $node_storage = \Drupal::entityManager()->getStorage('node');
//    $node_storage->load($todo['eid']);
    //TODO rework this so other entities can be published/unpublished too.

    $entity = \Drupal\node\Entity\Node::load($todo['eid']);
    switch ($todo['action']) {
      case ASF_ACTION_PUBLISH:
        $this->doPublishAction($entity);
        break;

      case ASF_ACTION_UNPUBLISH:
        $this->doUnpublishAction($entity);
        break;
    }
  }

  /**
   * Unpublish one item.
   */
  function doUnpublishAction($entity) {
    if ($entity->isPublished()) {
      try {
        $entity->setPublished(0);
        $entity->save();
        \Drupal::logger('asf')
          ->notice('Entities unpublished: <pre>' . print_r($entity->id(), TRUE) . '</pre>');
      } catch (Exception $e) {
        \Drupal::logger('asf')
          ->notice('Entities could not be unpublished: <pre>' . print_r($e, TRUE) . '</pre>');
      }

    }
  }

  /**
   * Publish one item
   * @param $entity
   */
  function doPublishAction($entity) {
    if (!$entity->isPublished()) {
      try {
        $entity->setPublished(1);
        $entity->save();

        \Drupal::logger('asf')
          ->notice('Entities published: <pre>' . print_r($entity->id(), TRUE) . '</pre>');
      } catch (Exception $e) {
        \Drupal::logger('asf')
          ->notice('Entities could not be published: <pre>' . print_r($e, TRUE) . '</pre>');
      }
    }
  }
}